@@ -52,7 +52,7 @@ use llvm_sys::{LLVMAttributeFunctionIndex, LLVMInlineAsmDialect::*};
 use std::ffi::{CStr, CString};
 use std::os::raw::c_char;
 use std::path::Path;
-use std::process::Command;
+use std::process::{Command, Stdio};
 use std::ptr;
 
 pub unsafe fn init() {
@@ -137,19 +137,82 @@ pub unsafe fn compile(input: &Path, output: &Path, bc_output: Option<&Path>) ->
     ret
 }
 
-/// Strip debug sections from resulting ELF relocatable file
+/// Get section names of functions
+///
+/// Only functions that do not belong to the default .text section have
+/// particular section names. So thd default `.text` won't be included in the
+/// result vector.
+pub(crate) unsafe fn get_function_section_names(bc: &Path) -> Result<Vec<String>> {
+    let mut section_names = vec![];
+    let context = LLVMGetGlobalContext();
+    let module = load_module(context, bc)?;
+    let mut func = LLVMGetFirstFunction(module);
+    while !func.is_null() {
+        let secptr = LLVMGetSection(func);
+        if !secptr.is_null() {
+            let secname = CStr::from_ptr(secptr).to_string_lossy().into_owned();
+            section_names.push(secname);
+        }
+        func = LLVMGetNextFunction(func);
+    }
+    LLVMDisposeModule(module);
+
+    Ok(section_names)
+}
+
+/// Strip unnecessary sections from resulting ELF relocatable file
 ///
 /// This removes sections of which name start with `.debug` and their
 /// associated relocation sections. But .BTF related sections are not stripped.
 ///
-/// cf) `llvm_sys::debuginfo::LLVMStripModuleDebugInfo` removes BTF sections so do not call it.
-pub(crate) fn strip_debug(target: &impl AsRef<Path>) -> Result<()> {
-    Command::new("llvm-strip-12")
-        .arg("-g")
+/// cf) `llvm_sys::debuginfo::LLVMStripModuleDebugInfo` removes BTF sections so
+/// do not call it.
+///
+/// .text section is also removed.
+///
+pub(crate) fn strip_unnecessary(target: &impl AsRef<Path>, delete_btf: bool) -> Result<()> {
+    let cmd = [
+        "llvm-strip",
+        "llvm-strip-13",
+        "llvm-strip-12",
+        "llvm-strip-11",
+    ]
+    .iter()
+    .find(|cmd| {
+        Command::new(cmd)
+            .arg("--version")
+            .stdout(Stdio::null())
+            .status()
+            .is_ok()
+    })
+    .ok_or_else(|| anyhow!("llvm-strip command not found"))?;
+
+    Command::new(cmd)
+        .arg("--strip-debug")
         .arg(target.as_ref())
         .status()
         .map(|_| ())
-        .or_else(|e| Err(anyhow!("llvm-strip failed: {}", e)))
+        .or_else(|e| Err(anyhow!("llvm-strip --strip-debug failed: {}", e)))?;
+
+    // Even if there does not exist any function in .text section, .text
+    // section is created with zero size as a result of compilation. So it is
+    // needed to remove it explictly.
+    let mut cmd = Command::new(cmd);
+    if delete_btf {
+        // The BTF section generated by rustc contains characters that are not
+        // permitted by BPF verifier of the Linux kernel. So those characters
+        // should be fixed before loading the BTF sections. But some utils that
+        // are not aware of rustc generated BTF such as `tc` do not handle
+        // this. In this case just remove BTF sections to avoid BPF verifier
+        // problem.
+        cmd.args("--remove-section .BTF.ext --remove-section .BTF".split(' '));
+    }
+    cmd.args("--remove-section .text".split(' '))
+        .arg("--no-strip-all")
+        .arg(target.as_ref())
+        .status()
+        .map(|_| ())
+        .or_else(|e| Err(anyhow!("llvm-strip --remove-section .text failed: {}", e)))
 }
 
 pub unsafe fn process_ir(context: LLVMContextRef, module: LLVMModuleRef) -> Result<()> {
